@@ -127,18 +127,16 @@ class Parser:
             self.process_set(tables, set_line)
             set_line = None
         return set_line, set_was_in_line
-        
+
     def check_new_statement_start(self, line: str, statement: str) -> bool:
         new_statement = False
-        if statement and (('(' in statement and ')' not in statement) or (
-            ')' and '(' not in statement)):
-            
+        if statement and statement.count("(") == statement.count(")"):
             new_statements_tokens = ["ALTER ", "CREATE ", "DROP ", "SET "]
             for key in new_statements_tokens:
                 if line.upper().startswith(key):
                     new_statement = True
         return new_statement
-        
+
     def parse_data(self):  # noqa: C901 need to refactor this
         tables = []
         block_comments = []
@@ -159,7 +157,7 @@ class Parser:
             if line or num == len(lines) - 1:
                 # to avoid issues when comma or parath are glued to column name
                 new_statement = self.check_new_statement_start(line, statement)
-            
+
                 final_line = line.endswith(";")
 
                 if not skip and not set_was_in_line and not new_statement:
@@ -177,6 +175,7 @@ class Parser:
 
                 self.set_default_flags_in_lexer()
                 if not set_line and statement:
+
                     self.parse_statement(tables, statement)
                 if new_statement:
                     statement = line
